// a few routines still required by the compps.f and comprefl.f models which have not
// been converted over to MZCompRefl objects yet.

#include <functionMap.h>
#include <xsFortran.h>
#include <FunctionUtility.h>
#include <XSUtil/Utils/XSutility.h>
#include <cfortran.h>

// Functions called FROM here to Fortran:

PROTOCCALLSFSUB6(POWREFJ,powrefj,FLOATV,FLOATV,INT,FLOATV,FLOATV,FLOATV)
#define POWREFJ(params,x,jmax,sptot,sppow,spref) \
          CCALLSFSUB6(POWREFJ,powrefj,FLOATV,FLOATV,INT,FLOATV,FLOATV,FLOATV, \
          params,x,jmax,sptot,sppow,spref) 

   // These take too many parameters for cfortran.h.  With its current settings the upper limit is 14.                 
extern "C" float clcwrm_(float& X0,float& Gamma,float& Temp,float& Xi,float& Ab_,float& Ab0,int& Firstflag,int& qHHe,
                        float* sigma,float* ion,float* integ,float* arec,double* num,double* ratio,int* z,float* aw,
                        float* abo,float* e,int* iab0,int* imax);

// Functions called from Fortran TO here:

float sigmabfj(int In, float* X, int N, float Gamma0, float Temp0,
               float Xi0, float Ab_0, float Ab0, float* Asc0);
FCALLSCFUN9(FLOAT,sigmabfj,SIGMABFJ,sigmabfj,INT,FLOATV,INT,FLOAT,FLOAT,FLOAT,FLOAT,FLOAT,PFLOAT)                
                

float warmabj(float X0, float Gamma, float Temp, float Xi, float Ab_, 
              float Ab0, int* firstflag, int qHHe);
              
void warmabCommon(float **pSigma, float **pIon, int **pZ, float **pAw, float **pAo,
                float **pE, int **pIab0, int **pImax);


float sigmabfj(int In, float* X, int N, float Gamma0, float Temp0,
                     float Xi0, float Ab_0, float Ab0, float* Asc0)
{
   // Translated and still called from Fortran.  ASSUME In is a 1-based
   // Fortran array index.
   
   //  bound-free cross section for ionized matter with cosmic abundances
   //  and variable iron abundance (in units of solar iron abundance)
   //  per hydrogen atom in units of the Thomson cross section
   //  e is in units of 511 keV
   
   using namespace XSutility;
   static bool fl1 = true;
   static int firstflag = 1;
   static string fgsolr0;
   static auto_array_ptr<float> apXold(0);
   static auto_array_ptr<float> apOpac(0);
   
   // In this version, changes of ear during one xspec session are allowed.
   if (fgsolr0 != FunctionUtility::ABUND())
   {
      fl1 = true;
      fgsolr0 = FunctionUtility::ABUND();
   }
   
   static float gamma=.0, temp=.0, xi=.0, ab_=99999., ab=99999., asc=.0;
   if (gamma != Gamma0 || temp != Temp0 || xi != Xi0 || ab_ != Ab_0 || ab != Ab0)
   {
      gamma = Gamma0;
      temp = Temp0;
      xi = Xi0;
      ab_ = Ab_0;
      ab = Ab0;
      firstflag = 1;
      fl1 = true;
      float xas = .01947;
      asc = 1.503e2*warmabj(xas,gamma,temp,xi,ab_,ab,&firstflag,0)*xas*xas*xas;
   }
   
   static int nOld = 0;
   float *xold = apXold.get();
   if (!xold || nOld != N || X[In-1] != xold[In-1])
   {
      apOpac.reset(new float[N]);
      apXold.reset(new float[N]);
      xold = apXold.get();
      nOld = N;
      for (int i=0; i<N; ++i)
         xold[i] = X[i];
      fl1 = true;
   }
   
   float *opac = apOpac.get();
   if (fl1)
   {
      for (int i=0; i<N; ++i)
         opac[i] = 1.503e2*warmabj(X[i],gamma,temp,xi,ab_,ab,&firstflag,0);
      fl1 = false;
   }
   
   *Asc0 = asc;
   return opac[In-1];
}

float warmabj(float X0, float Gamma, float Temp, float Xi, float Ab_, 
                float Ab0, int* firstflag, int qHHe)
{
   using namespace XSutility;
   
   static auto_array_ptr<float> apInt(new float[20*30]);
   static auto_array_ptr<float> apArec(new float[20*30]);
   static auto_array_ptr<double> apNum(new double[20*30]);
   static auto_array_ptr<double> apRatio(new double[20*30]);
   
   float *sigma=0;
   float *ion=0;
   int *z=0;
   float *aw=0;
   float *abo=0;
   float *e=0;
   int* iab0=0;
   int* imax=0;
   warmabCommon(&sigma, &ion, &z, &aw, &abo, &e, &iab0, &imax);
      
   return clcwrm_(X0,Gamma,Temp,Xi,Ab_,Ab0,*firstflag,qHHe,sigma,
                ion,apInt.get(),apArec.get(),apNum.get(),apRatio.get(),
                z, aw, abo, e, iab0, imax);
}

void warmabCommon(float **pSigma, float **pIon, int **pZ, float **pAw, float **pAbo,
                float **pE, int **pIab0, int **pImax)
{
   // Memory shared between warmabj and warmaby (called by xsabsori.f)
   static XSutility::auto_array_ptr<float> apSigma(new float[20*30*800]);
   static XSutility::auto_array_ptr<float> apIon(new float[20*30*10]);
   static int z[20], iab0=0, imax=0;
   static float aw[20], abo[20], e[800];
   
   *pSigma = apSigma.get();
   *pIon = apIon.get();
   *pZ = z;
   *pAw = aw;
   *pAbo = abo;
   *pE = e;
   *pIab0 = &iab0;
   *pImax = &imax;
}

